(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var ECMAScript = Package.ecmascript.ECMAScript;
var _ = Package.underscore._;
var AppConfig = Package['application-configuration'].AppConfig;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;

/* Package-scope variables */
var SandstormEmail;

(function(){

///////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                   //
// packages/sandstorm-email/email.js                                                                 //
//                                                                                                   //
///////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                     //
var Future = Npm.require("fibers/future");                                                           // 1
var urlModule = Npm.require("url");                                                                  // 2
var MailComposer = Npm.require("mailcomposer").MailComposer;                                         // 3
                                                                                                     //
SandstormEmail = {};                                                                                 // 5
                                                                                                     //
var getSmtpUrl = function () {                                                                       // 7
  var setting = Settings.findOne({ _id: "smtpUrl" });                                                // 8
  if (setting) {                                                                                     // 9
    return setting.value;                                                                            // 10
  } else {                                                                                           //
    return process.env.MAIL_URL;                                                                     // 12
  }                                                                                                  //
};                                                                                                   //
                                                                                                     //
var makePool = function (mailUrlString) {                                                            // 16
  var mailUrl = urlModule.parse(mailUrlString);                                                      // 17
  if (mailUrl.protocol !== "smtp:") throw new Error("Email protocol in $MAIL_URL (" + mailUrlString + ") must be 'smtp'");
                                                                                                     //
  var port = +mailUrl.port;                                                                          // 22
  var auth = false;                                                                                  // 23
  if (mailUrl.auth) {                                                                                // 24
    var parts = mailUrl.auth.split(":", 2);                                                          // 25
    auth = { user: parts[0],                                                                         // 26
      pass: parts[1] };                                                                              // 27
  }                                                                                                  //
                                                                                                     //
  var simplesmtp = Npm.require("simplesmtp");                                                        // 30
  var pool = simplesmtp.createClientPool(port, // Defaults to 25                                     // 31
  mailUrl.hostname, // Defaults to "localhost"                                                       // 33
  { secureConnection: port === 465,                                                                  // 34
    // XXX allow maxConnections to be configured?                                                    //
    auth: auth });                                                                                   // 36
                                                                                                     //
  pool._futureWrappedSendMail = _.bind(Future.wrap(pool.sendMail), pool);                            // 38
  return pool;                                                                                       // 39
};                                                                                                   //
                                                                                                     //
// We construct smtpPool at the first call to Email.send, so that                                    //
// Meteor.startup code can set $MAIL_URL.                                                            //
var pool = undefined;                                                                                // 44
var configured = false;                                                                              // 45
                                                                                                     //
Meteor.startup(function () {                                                                         // 47
  Settings.find({ _id: "smtpUrl" }).observeChanges({                                                 // 48
    removed: function () {                                                                           // 49
      configured = false;                                                                            // 50
    },                                                                                               //
                                                                                                     //
    changed: function () {                                                                           // 53
      configured = false;                                                                            // 54
    },                                                                                               //
                                                                                                     //
    added: function () {                                                                             // 57
      configured = false;                                                                            // 58
    }                                                                                                //
  });                                                                                                //
                                                                                                     //
  // Accounts.emailToken is set to use "Email" by default. Change it to use our mail service.        //
  Accounts.emailToken.setEmailPackage("SandstormEmail");                                             // 63
});                                                                                                  //
                                                                                                     //
var getPool = function (smtpUrl) {                                                                   // 66
  if (smtpUrl) {                                                                                     // 67
    return makePool(smtpUrl);                                                                        // 68
  } else if (!configured) {                                                                          //
    configured = true;                                                                               // 70
    var url = getSmtpUrl();                                                                          // 71
    if (url) {                                                                                       // 72
      pool = makePool(url);                                                                          // 73
    }                                                                                                //
  }                                                                                                  //
                                                                                                     //
  return pool;                                                                                       // 77
};                                                                                                   //
                                                                                                     //
var nextDevmodeMailId = 0;                                                                           // 80
var outputStream = process.stdout;                                                                   // 81
                                                                                                     //
var devModeSend = function (mc) {                                                                    // 83
  // This seems to be dead code.                                                                     //
  var devmodeMailId = nextDevmodeMailId++;                                                           // 85
                                                                                                     //
  var stream = outputStream;                                                                         // 87
                                                                                                     //
  // This approach does not prevent other writers to stdout from interleaving.                       //
  stream.write("====== BEGIN MAIL #" + devmodeMailId + " ======\n");                                 // 90
  stream.write("(Mail not sent; to enable sending, set the MAIL_URL " + "environment variable.)\n");
  mc.streamMessage();                                                                                // 93
  mc.pipe(stream, { end: false });                                                                   // 94
  var future = new Future();                                                                         // 95
  mc.on("end", function () {                                                                         // 96
    stream.write("====== END MAIL #" + devmodeMailId + " ======\n");                                 // 97
    future["return"]();                                                                              // 98
  });                                                                                                //
                                                                                                     //
  future.wait();                                                                                     // 101
};                                                                                                   //
                                                                                                     //
var smtpSend = function (pool, mc) {                                                                 // 104
  pool._futureWrappedSendMail(mc).wait();                                                            // 105
};                                                                                                   //
                                                                                                     //
// Old comment below                                                                                 //
/**                                                                                                  //
 * Send an email.                                                                                    //
 *                                                                                                   //
 * Connects to the mail server configured via the MAIL_URL environment                               //
 * variable. If unset, prints formatted message to stdout. The "from" option                         //
 * is required, and at least one of "to", "cc", and "bcc" must be provided;                          //
 * all other options are optional.                                                                   //
 *                                                                                                   //
 * @param options                                                                                    //
 * @param options.from {String} RFC5322 "From:" address                                              //
 * @param options.to {String|String[]} RFC5322 "To:" address[es]                                     //
 * @param options.cc {String|String[]} RFC5322 "Cc:" address[es]                                     //
 * @param options.bcc {String|String[]} RFC5322 "Bcc:" address[es]                                   //
 * @param options.replyTo {String|String[]} RFC5322 "Reply-To:" address[es]                          //
 * @param options.subject {String} RFC5322 "Subject:" line                                           //
 * @param options.text {String} RFC5322 mail body (plain text)                                       //
 * @param options.html {String} RFC5322 mail body (HTML)                                             //
 * @param options.headers {Object} custom RFC5322 headers (dictionary)                               //
 */                                                                                                  //
                                                                                                     //
// New API doc comment below                                                                         //
/**                                                                                                  //
 * @summary Send an email. Throws an `Error` on failure to contact mail server                       //
 * or if mail server returns an error. All fields should match                                       //
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                                      //
 * @locus Server                                                                                     //
 * @param {Object} options                                                                           //
 * @param {String} options.from "From:" address (required)                                           //
 * @param {String|String[]} options.to,cc,bcc,replyTo                                                //
 *   "To:", "Cc:", "Bcc:", and "Reply-To:" addresses                                                 //
 * @param {String} [options.subject]  "Subject:" line                                                //
 * @param {String} [options.text|html] Mail body (in plain text or HTML)                             //
 * @param {Object} [options.headers] Dictionary of custom headers                                    //
 * @param {String} [options.smtpUrl] SMTP server to use. Otherwise defaults to configured one.       //
 * @param {Object} [options.attachments] Attachments. See:                                           //
 *   https://github.com/nodemailer/mailcomposer/tree/v0.1.15#add-attachments                         //
 * @param {String} [options.envelopeFrom] Envelope sender.                                           //
 */                                                                                                  //
SandstormEmail.send = function (options) {                                                           // 147
  var mc = new MailComposer();                                                                       // 148
                                                                                                     //
  // setup message data                                                                              //
  mc.setMessageOption({                                                                              // 151
    from: options.from,                                                                              // 152
    to: options.to,                                                                                  // 153
    cc: options.cc,                                                                                  // 154
    bcc: options.bcc,                                                                                // 155
    replyTo: options.replyTo,                                                                        // 156
    subject: options.subject,                                                                        // 157
    text: options.text,                                                                              // 158
    html: options.html                                                                               // 159
  });                                                                                                //
                                                                                                     //
  if (options.envelopeSender) {                                                                      // 162
    var envelope = mc.getEnvelope();                                                                 // 163
    envelope.from = options.envelopeFrom;                                                            // 164
    mc.setMessageOption({                                                                            // 165
      envelope: envelope                                                                             // 166
    });                                                                                              //
  }                                                                                                  //
                                                                                                     //
  _.each(options.headers, function (value, name) {                                                   // 170
    mc.addHeader(name, value);                                                                       // 171
  });                                                                                                //
                                                                                                     //
  _.each(options.attachments, function (value) {                                                     // 174
    mc.addAttachment(value);                                                                         // 175
  });                                                                                                //
                                                                                                     //
  SandstormEmail.rawSend(mc, options.smtpUrl);                                                       // 178
};                                                                                                   //
                                                                                                     //
/**                                                                                                  //
 * @summary Sends a raw email with a MailComposer object.                                            //
 * Throws an `Error` on failure to contact mail server                                               //
 * or if mail server returns an error. All fields should match                                       //
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                                      //
 * @locus Server                                                                                     //
 * @param {Object} mc A MailCompser object that you wish to send                                     //
 * @param {String} smtpUrl SMTP server to use. If falsey, defaults to configured one.                //
*/                                                                                                   //
SandstormEmail.rawSend = function (mc, smtpUrl) {                                                    // 190
  // SimpleSmtp does not add leading dots, so we need to.                                            //
  // See http://tools.ietf.org/html/rfc5321#section-4.5.2                                            //
  mc._message.body = mc._message.body.replace(/(^|\n)\./g, "$1..");                                  // 193
                                                                                                     //
  var pool = getPool(smtpUrl);                                                                       // 195
  if (pool) {                                                                                        // 196
    smtpSend(pool, mc);                                                                              // 197
  } else {                                                                                           //
    throw new Error("SMTP pool is misconfigured.");                                                  // 199
  }                                                                                                  //
};                                                                                                   //
///////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-email'] = {
  SandstormEmail: SandstormEmail
};

})();

//# sourceMappingURL=sandstorm-email.js.map
